#include <Adafruit_GFX.h>
#include <SPI.h>
#include <Wire.h>
//#include <Adafruit_ILI9341.h>

#include <Optimized_ILI9341.h>
#include <Adafruit_STMPE610.h>

// This is calibration data for the raw touch data to the screen coordinates
#define TS_MINX 150
#define TS_MINY 130
#define TS_MAXX 3800
#define TS_MAXY 4000

#define STMPE_CS 8
Adafruit_STMPE610 ts = Adafruit_STMPE610(STMPE_CS);

#define TFT_CS 10
#define TFT_DC 9
#define TFT_RST -1
Optimized_ILI9341 tft = Optimized_ILI9341(TFT_CS, TFT_DC);

#define BOXSIZE 33
int oldcolor, currentcolor;
boolean RecordOn = false;
int x = 0;

uint8_t uCurrentScreen = 0;
enum {
  SCREEN_MAIN=0, SCREEN_SECOND, SCREEN_THIRD};

//-----------------------------------------------------------
//
//-----------------------------------------------------------

void setup(void)
{
  delay(2000);
  Serial.begin(9600);
  tft.begin();
  
  // hack wait for all outputs to complete clear out any pending reads...
  // clear out any current received bytes
  uint16_t wTimeout = 0xffff;
  uint8_t r;
  while (((SPI0.SR) & (15 << 12)) && (--wTimeout)) ; // wait until empty
  wTimeout = 0x10;    // should not go more than 4...
  while ((((SPI0.SR) >> 4) & 0xf) && (--wTimeout))  {
    r = SPI0.POPR;
  }

  if (!ts.begin()) { 
    Serial.println("Unable to start touchscreen.");
    delay(2000);
  } 
  else { 
    Serial.println("Touchscreen started."); 
    delay(2000);
  }
  
  tft.setRotation(3);
  Serial.println("Rotation set to 3 Landscape"); 
  SetDisplayMainScreen();

}
//-----------------------------------------------------------
//
//-----------------------------------------------------------

void loop()
{
  if (ts.bufferEmpty()) {
    delay(10);
    return;
  }
  
  switch (uCurrentScreen) {
  case SCREEN_MAIN: 
    ProcessTouchMainScreen();
    break;
  case SCREEN_SECOND: 
    ProcessTouchSecondScreen();
    break;
  case SCREEN_THIRD: 
    ProcessTouchThirdScreen();
    break;
  }
}  

//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------

void SetDisplayMainScreen(){

  int i;

  tft.fillScreen(ILI9341_BLACK);
  Serial.println("BLACK Main Screen Selected");
 
  tft. setTextColor(ILI9341_BLUE);
   tft.setTextSize(3);

  for(i=0; i<=7; i++)
  {
    tft.fillRoundRect(((i*6)+7)+(i*BOXSIZE), 7, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
    tft.setCursor(((i*6)+16)+(i*BOXSIZE), 13);
    tft.print(i);
  }
  for(i=0; i<=7; i++)
  {
    tft.fillRoundRect(((i*6)+7)+(i*BOXSIZE), 46, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
    tft.setCursor(((i*6)+16)+(i*BOXSIZE), 52);
    tft.print(i+8,HEX);
  }
  tft.fillRoundRect(280, 200, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
  tft.fillTriangle(290, 207, 290, 227, 305, 217, ILI9341_BLUE); 

  tft.fillRoundRect(7, 200, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
  tft.fillTriangle(29, 207, 29, 227, 13, 217, ILI9341_BLUE); 

  // Set which screen is current
  uCurrentScreen = SCREEN_MAIN;
  ClearTouchBuffer();

}
//----------------------------------------------------------------------------
// Draw a red frame while a button is touched
//----------------------------------------------------------------------------
void SelectedBtn(int x1, int y1, int x2, int y2)
{
  tft.drawRoundRect (x1, y1, x2, y2, 7, ILI9341_RED);
  tft.drawRoundRect (x1+1, y1+1, x2-2, y2-2, 7, ILI9341_RED);
  tft.drawRoundRect (x1+2, y1+2, x2-4, y2-4, 7, ILI9341_RED);

  while (!ts.bufferEmpty() || ts.touched()) {
    TS_Point p = ts.getPoint();
    delay(10);
  }

  tft.drawRoundRect (x1, y1, x2, y2, 7, ILI9341_WHITE); 
  tft.drawRoundRect (x1+1, y1+1, x2-2, y2-2, 7, ILI9341_WHITE);
  tft.drawRoundRect (x1+2, y1+2, x2-4, y2-4, 7, ILI9341_WHITE);

  while (!ts.bufferEmpty()) {
    TS_Point p = ts.getPoint();
    delay(10);
  }
 
}
//----------------------------------------------------------------------------
// Move the slider Knob
//----------------------------------------------------------------------------
void MoveKnob(int x1, int y1, int x2, int y2)
{   

  tft.fillRect(150,0,30,300,ILI9341_BLUE);
  tft.drawRect(160,10,10,180,ILI9341_BLACK);
  tft.fillRect(150,y1,30,15,ILI9341_WHITE);
  Serial.print("Knob Location  ");
  Serial.println(y1);

}
//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------

void SetDisplayThirdScreen(){  

  tft.fillScreen(ILI9341_WHITE);
  Serial.println("WHITE Third Screen Selected");
  
  tft.fillRoundRect(7, 200, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
  tft.fillTriangle(29, 207, 29, 227, 13, 217, ILI9341_WHITE); 
  tft.fillRoundRect(280, 200, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
  tft.fillTriangle(290, 207, 290, 227, 305, 217, ILI9341_WHITE); 

  uCurrentScreen = SCREEN_THIRD;
  ClearTouchBuffer();
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
TS_Point GetAndMapTouchPoint(uint8_t iDisplay) {
   TS_Point  p = ts.getPoint();

  uint16_t t = map(p.y, TS_MINY, TS_MAXY,  tft.width(), 0);
  p.y = map(p.x, TS_MINX, TS_MAXX, 0, tft.height());
  p.x = t;  // X and Y now swapped...

#if 1
  Serial.print("Privious Display: ");
  Serial.print(iDisplay, DEC);
  Serial.print(" X: ");
  Serial.print(p.x, DEC);
  Serial.print(" Y: ");
  Serial.println(p.y, DEC);
#endif

  return p;
}

//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ProcessTouchThirdScreen(){  
  int i;

  TS_Point p = GetAndMapTouchPoint(3);
  
  if (p.y >= 200 && p.y <= (200+BOXSIZE)) {
     if (p.x >= 7 && p.x <= (7+BOXSIZE)) { 
        SelectedBtn(7, 200, BOXSIZE, BOXSIZE);
        SetDisplaySecondScreen();
     }
     if (p.x >= 280 && p.x <= (280+BOXSIZE)) { 
        SelectedBtn(280, 200, BOXSIZE, BOXSIZE);
        SetDisplayMainScreen();
        return;
     }
  }
}
//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------

void SetDisplaySecondScreen(){  

  tft.fillScreen(ILI9341_BLUE);
  Serial.println("BLUE Second Screen Selected");
  
  tft.drawRect(160,10,10,180,ILI9341_BLACK);
  tft.fillRect(150,100,30,15,ILI9341_WHITE);
  tft.fillRoundRect(7, 200, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
  tft.fillTriangle(29, 207, 29, 227, 13, 217, ILI9341_BLUE); 
  tft.fillRoundRect(280, 200, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
  tft.fillTriangle(290, 207, 290, 227, 305, 217, ILI9341_BLUE); 

  uCurrentScreen = SCREEN_SECOND;
  ClearTouchBuffer();
}

//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ClearTouchBuffer() {
  while (!ts.bufferEmpty()) {
    TS_Point p = ts.getPoint();
    delay(10);
  }
 
}

//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ProcessTouchSecondScreen(){  
  int i;
  TS_Point p = GetAndMapTouchPoint(2);

  if ((p.x >= 150 && p.x <= (150+30))&&(p.y >= 10 && p.y <= (10+180))){
       MoveKnob(150,p.y,30,15);
  }
  else if (p.y >= 200 && p.y <= (200+BOXSIZE)) {
    
    if (p.x >= 7 && p.x <= (7+BOXSIZE)) { 
      SelectedBtn(7, 200, BOXSIZE, BOXSIZE);
      SetDisplayMainScreen();
    }
    if (p.x >= 280 && p.x <= (280+BOXSIZE)) { 
      SelectedBtn(280, 200, BOXSIZE, BOXSIZE);
      SetDisplayThirdScreen();
      return;
    }
  }
}

//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ProcessTouchMainScreen(){

  // Retrieve a point  
  TS_Point p = GetAndMapTouchPoint(1);
  uint8_t i;
  uint16_t xButtonStart;

  // Check if first row has been selected
  if (p.y >= 7 && p.y <= (7+BOXSIZE)) {
    for(i=0; i<=7; i++)
    {
      xButtonStart = 7 + i*(BOXSIZE + 6);
      if (p.x >= xButtonStart && p.x <= (xButtonStart + BOXSIZE)) {
        SelectedBtn(xButtonStart, 7, BOXSIZE, BOXSIZE);
        break;
      }
    }
  }

  // Check if second row was selected
  if (p.y >= 46 && p.y <= (46+BOXSIZE)) {
    for(i=0; i<=7; i++)
    {
      xButtonStart = 7 + i*(BOXSIZE + 6);
      if (p.x >= xButtonStart && p.x <= (xButtonStart + BOXSIZE)) {
        SelectedBtn(xButtonStart, 46, BOXSIZE, BOXSIZE);
        break;
      }
    }
  }

  // Check page display buttons
  if (p.y >= 200 && p.y <= (200+BOXSIZE)) {
    //RestoreSPIRegs(SPIR_TFT);
    //oldcolor = currentcolor;
    if (p.x >= 7 && p.x <= (7+BOXSIZE)) { 
      //currentcolor = ILI9341_RED; 
      SelectedBtn(7, 200, BOXSIZE, BOXSIZE);
      SetDisplayThirdScreen();
    }
    if (p.x >= 280 && p.x <= (280+BOXSIZE)) { 
      //currentcolor = ILI9341_BLACK;
      SelectedBtn(280, 200, BOXSIZE, BOXSIZE);
      SetDisplaySecondScreen();
      return;
    }
  }
}










